/*!
 * \file move.c
 * \brief Move Source File
 * \authors Tin
 * \version 1
 * \date 25/03/2021
 *
 * Movements functions implementation.
 *
 */

#include <SDL2/SDL.h>
#include<stdio.h>
#include "move.h"
#include "personnage.h"
#include "collision.h"
#include "prof.h"
#include "DS.h"
#include "GAME.h"
#include "map.h"

/*!
 * \fn void move_hero_4_teacher(Personnage * hero, SDL_Rect * hero_rect, Prof * teacher, Prof * teacher2, Prof * teacher3, Prof * teacher4, DS * exam, MAP * map_game, int *DIE, int * LEVELUP)
 * \brief This function moves the hero following his position on the map, it will change the DIE value of the LEVELUP value if necessary.
 *
 * \param [in] hero The hero pointer
 * \param [in] hero_rect The pointer to structure rect of hero
 * \param [in] teacher Pointer of the first teacher
 * \param [in] teacher2 Pointer of the second teacher
 * \param [in] teacher3 Pointer of the third teacher
 * \param [in] teacher4 Pointer of the fourth teacher
 * \param [in] exam The exam pointer
 * \param [in] map_game The map_game pointer
 * \param [in] DIE Pointer to an integer, 0 if not dead, 1 if dead
 * \param [in] LEVELUP Pointer to an integer, 1 if level up, 0 if not.
 */

void move_hero_4_teacher(Personnage * hero, SDL_Rect * hero_rect, Prof * teacher, Prof * teacher2, Prof * teacher3, Prof * teacher4, DS * exam, MAP * map_game, int *DIE, int * LEVELUP)
{
	int collision1,collision2,collision3,collision4; //test with teacher 1 2 3 
    //collision variable
    int collision;


    switch (hero->animation.direction_personnage) {

        case 1:
            hero->y_pers -= SPEED;
            break;
        case 2:
            hero->y_pers += SPEED;
            break;
        case 3:
            hero->x_pers -= SPEED;
            break;
        case 4:
            hero->x_pers += SPEED;
            break;
        default:
            break;
    }
    if (hero->animation.direction_personnage!=0){
        hero->animation.index_image++;
        if (hero->animation.index_image > 11) {
            hero->animation.index_image = 0;
        }
    }
    //recalculate 4 points of hero
    calcul_points_pers(hero);

    // collision detection
    collision1 = test_position(hero, teacher, exam,
                              map_game); // return 0 if nothing, 1 if wall, 2 if exam, 3 if prof, 4 if bounds
	collision2 = test_position(hero, teacher2, exam,
                              map_game); // return 0 if nothing, 1 if wall, 2 if exam, 3 if prof, 4 if bounds
	collision3 = test_position(hero, teacher3, exam,
                              map_game); // return 0 if nothing, 1 if wall, 2 if exam, 3 if prof, 4 if bounds
	collision4 = test_position(hero, teacher4, exam,
                              map_game); // return 0 if nothing, 1 if wall, 2 if exam, 3 if prof, 4 if bounds



	if(collision1 == 3 || collision2 == 3 || collision3 == 3 || collision4 == 3) collision = 3;
	
	else collision = collision4;
	
    switch (collision)
    {
        case 0:
            hero_rect->x = (int) hero->x_pers;
            hero_rect->y = (int) hero->y_pers;
            break;
        case 1:
            hero->y_pers = hero_rect->y;
            hero->x_pers = hero_rect->x;
            break;
        case 2:
            *LEVELUP = 1;
            break;

        case 3:

            *DIE = 1;
            break;

        case 4:
            if (hero->x_pers <= 0) hero->x_pers = 2;
            if (hero->y_pers <= 0) hero-> y_pers = 2;
            if (hero->x_pers >= WINDOW_WIDTH - hero_rect->w) hero->x_pers = WINDOW_WIDTH - hero_rect->w - 2;
            if (hero->y_pers >= WINDOW_HEIGHT - hero_rect->h) hero->y_pers = WINDOW_HEIGHT - hero_rect->h - 2;
            hero_rect->x = (int) hero->x_pers;
            hero_rect->y = (int) hero->y_pers;
            break;
        default:
            break;
    }
}

/*!
 * \fn void move_hero_3_teacher(Personnage * hero, SDL_Rect * hero_rect, Prof * teacher, Prof * teacher2, Prof * teacher3, DS * exam, MAP * map_game, int *DIE, int * LEVELUP)
 * \brief This function moves the hero following his position on the map, it will change the DIE value of the LEVELUP value if necessary.
 *
 * \param [in] hero The hero pointer
 * \param [in] hero_rect The pointer to structure rect of hero
 * \param [in] teacher Pointer of the first teacher
 * \param [in] teacher2 Pointer of the second teacher
 * \param [in] teacher3 Pointer of the third teacher
 * \param [in] exam The exam pointer
 * \param [in] map_game The map_game pointer
 * \param [in] DIE Pointer to an integer, 0 if not dead, 1 if dead
 * \param [in] LEVELUP Pointer to an integer, 1 if level up, 0 if not.
 */

void move_hero_3_teacher(Personnage * hero, SDL_Rect * hero_rect, Prof * teacher,Prof * teacher2, Prof * teacher3, DS * exam, MAP * map_game, int *DIE, int * LEVELUP)
{
	int collision1,collision2,collision3; //test with teacher 1 2 3 
	
    //collision variable
    int collision;


    switch (hero->animation.direction_personnage) {

        case 1:
            hero->y_pers -= SPEED;
            break;
        case 2:
            hero->y_pers += SPEED;
            break;
        case 3:
            hero->x_pers -= SPEED;
            break;
        case 4:
            hero->x_pers += SPEED;
            break;
        default:
            break;

    }
    if (hero->animation.direction_personnage!=0){
        hero->animation.index_image++;
        if (hero->animation.index_image > 11) {
            hero->animation.index_image = 0;
        }
    }
    //recalculate 4 points of hero
    calcul_points_pers(hero);

    // collision detection
    collision1 = test_position(hero, teacher, exam,
                              map_game); // return 0 if nothing, 1 if wall, 2 if exam, 3 if prof, 4 if bounds

	collision2 = test_position(hero, teacher2, exam,
                              map_game); // return 0 if nothing, 1 if wall, 2 if exam, 3 if prof, 4 if bounds
							  
	collision3 = test_position(hero, teacher3, exam,
                              map_game); // return 0 if nothing, 1 if wall, 2 if exam, 3 if prof, 4 if bounds
							  
							  
	
	if(collision1 == 3 || collision2 == 3 || collision3 == 3) collision = 3;
	
	else collision = collision3;

	
    switch (collision)
    {
        case 0:
            hero_rect->x = (int) hero->x_pers;
            hero_rect->y = (int) hero->y_pers;
            break;
        case 1:
            hero->y_pers = hero_rect->y;
            hero->x_pers = hero_rect->x;
            break;
        case 2:
            *LEVELUP = 1;
            break;

        case 3:

            *DIE = 1;
            break;

        case 4:
            if (hero->x_pers <= 0) hero->x_pers = 2;
            if (hero->y_pers <= 0) hero-> y_pers = 2;
            if (hero->x_pers >= WINDOW_WIDTH - hero_rect->w) hero->x_pers = WINDOW_WIDTH - hero_rect->w - 2;
            if (hero->y_pers >= WINDOW_HEIGHT - hero_rect->h) hero->y_pers = WINDOW_HEIGHT - hero_rect->h - 2;
            hero_rect->x = (int) hero->x_pers;
            hero_rect->y = (int) hero->y_pers;
            break;
        default:
            break;
    }
}

/*!
 * \fn void move_hero_2_teacher(Personnage * hero, SDL_Rect * hero_rect, Prof * teacher, Prof * teacher2, DS * exam, MAP * map_game, int *DIE, int * LEVELUP)
 * \brief This function moves the hero following his position on the map, it will change the DIE value of the LEVELUP value if necessary.
 *
 * \param [in] hero The hero pointer
 * \param [in] hero_rect The pointer to structure rect of hero
 * \param [in] teacher Pointer of the first teacher
 * \param [in] teacher2 Pointer of the second teacher
 * \param [in] exam The exam pointer
 * \param [in] map_game The map_game pointer
 * \param [in] DIE Pointer to an integer, 0 if not dead, 1 if dead
 * \param [in] LEVELUP Pointer to an integer, 1 if level up, 0 if not.
 */

void move_hero_2_teacher(Personnage * hero, SDL_Rect * hero_rect, Prof * teacher, Prof * teacher2, DS * exam, MAP * map_game, int *DIE, int * LEVELUP)
{
	//collision variable
    int collision1,collision2; //test with teacher 1 and 2
    //collision variable
    int collision;


    switch (hero->animation.direction_personnage) {

        case 1:
            hero->y_pers -= SPEED;
            break;
        case 2:
            hero->y_pers += SPEED;
            break;
        case 3:
            hero->x_pers -= SPEED;
            break;
        case 4:
            hero->x_pers += SPEED;
            break;
        default:
            break;
    }
    if (hero->animation.direction_personnage!=0){
        hero->animation.index_image++;
        if (hero->animation.index_image > 11) {
            hero->animation.index_image = 0;
        }
    }
    //recalculate 4 points of hero
    calcul_points_pers(hero);

    // collision detection
    collision1 = test_position(hero, teacher, exam,
                              map_game); // return 0 if nothing, 1 if wall, 2 if exam, 3 if prof, 4 if bounds
	
	
	collision2 = test_position(hero, teacher2, exam,
                              map_game); // return 0 if nothing, 1 if wall, 2 if exam, 3 if prof, 4 if bounds
							  
							  
	if(collision1 == 3 || collision2 == 3) collision = 3;
	else collision = collision2;	
	
	
    switch (collision)
    {
        case 0:
            hero_rect->x = (int) hero->x_pers;
            hero_rect->y = (int) hero->y_pers;
            break;
        case 1:
            hero->y_pers = hero_rect->y;
            hero->x_pers = hero_rect->x;
            break;
        case 2:
            *LEVELUP = 1;
            break;

        case 3:

            *DIE = 1;
            break;

        case 4:
            if (hero->x_pers <= 0) hero->x_pers = 2;
            if (hero->y_pers <= 0) hero-> y_pers = 2;
            if (hero->x_pers >= WINDOW_WIDTH - hero_rect->w) hero->x_pers = WINDOW_WIDTH - hero_rect->w - 2;
            if (hero->y_pers >= WINDOW_HEIGHT - hero_rect->h) hero->y_pers = WINDOW_HEIGHT - hero_rect->h - 2;
            hero_rect->x = (int) hero->x_pers;
            hero_rect->y = (int) hero->y_pers;
            break;
        default:
            break;
    }
}

/*!
 * \fn void move_hero(Personnage * hero, SDL_Rect * hero_rect, Prof * teacher, DS * exam, MAP * map_game, int *DIE, int * LEVELUP)
 * \brief This function moves the hero following his position on the map, it will change the DIE value of the LEVELUP value if necessary
 *
 * \param [in] hero The hero pointer
 * \param [in] hero_rect The pointer to structure rect of hero
 * \param [in] teacher Pointer of the teacher
 * \param [in] exam The exam pointer
 * \param [in] map_game The map_game pointer
 * \param [in] DIE Pointer to an integer, 0 if not dead, 1 if dead
 * \param [in] LEVELUP Pointer to an integer, 1 if level up, 0 if not.
 */

void move_hero(Personnage * hero, SDL_Rect * hero_rect, Prof * teacher, DS * exam, MAP * map_game, int *DIE, int * LEVELUP)
{
    //collision variable
    int collision;


    switch (hero->animation.direction_personnage) {

        case 1:
            hero->y_pers -= SPEED;
            break;
        case 2:
            hero->y_pers += SPEED;
            break;
        case 3:
            hero->x_pers -= SPEED;
            break;
        case 4:
            hero->x_pers += SPEED;
            break;
        default:
            break;
    }
    if (hero->animation.direction_personnage!=0){
        hero->animation.index_image++;
        if (hero->animation.index_image > 11) {
            hero->animation.index_image = 0;
        }
    }
    //recalculate 4 points of hero
    calcul_points_pers(hero);

    // collision detection
    collision = test_position(hero, teacher, exam,
                              map_game); // return 0 if nothing, 1 if wall, 2 if exam, 3 if prof, 4 if bounds


    switch (collision)
    {
        case 0:
            hero_rect->x = (int) hero->x_pers;
            hero_rect->y = (int) hero->y_pers;
            break;
        case 1:
            hero->y_pers = hero_rect->y;
            hero->x_pers = hero_rect->x;
            break;
        case 2:
            *LEVELUP = 1;
            break;

        case 3:

            *DIE = 1;
            break;

        case 4:
            if (hero->x_pers <= 0)
                hero->x_pers = 2;
            else if (hero->x_pers >= WINDOW_WIDTH - hero_rect->w)
                hero->x_pers = WINDOW_WIDTH - hero_rect->w - 2;
            if (hero->y_pers <= 0)
                hero-> y_pers = 2;
            else if (hero->y_pers >= WINDOW_HEIGHT - hero_rect->h)
                hero->y_pers = WINDOW_HEIGHT - hero_rect->h - 2;
            hero_rect->x = hero->x_pers;
            hero_rect->y = hero->y_pers;
            break;
        default:
            break;
    }
}

/*!
 * \fn void prof_move(Prof *prof, int * predirection, SDL_Rect * prof_rect, MAP * map, DS *exam)
 * \brief This function moves the teacher randomly.
 *
 * \param [in] prof The prof pointer
 * \param [in] predirection The direction pointer
 * \param [in] prof_rect The pointer to structure rect of prof
 * \param [in] map_game The map_game pointer
 * \param [in] exam The exam pointer
 */

void prof_move(Prof *prof, int * predirection, SDL_Rect * prof_rect, MAP * map, DS *exam){
    int randdirect;
    switch (*predirection)
    {
        case 0:
			prof->animation.direction_teacher = 0;
            prof->y_prof -= prof->speed_prof;
            break;
        case 1:
			prof->animation.direction_teacher = 1;
            prof->y_prof += prof->speed_prof;
            break;
        case 2:
			prof->animation.direction_teacher = 2;
            prof->x_prof -= prof->speed_prof;
            break;
        case 3:
			prof->animation.direction_teacher = 3;
            prof->x_prof += prof->speed_prof;
            break;
    }
    calcul_points_prof(prof);
    // prof collision detection
    int collision = test_position_prof(prof,map,exam); // return 0 if nothing, 1 if wall or bounds
    // set the positions following collision variable
	
	prof->animation.index_image++;
        if (prof->animation.index_image > 23) {
            prof->animation.index_image = 0;
        }
    switch (collision)
    {
        case 0:
            prof_rect->x = (int) prof->x_prof;
            prof_rect->y = (int) prof->y_prof;
            break;
        case 1:
            prof->y_prof = prof_rect->y;
            prof->x_prof = prof_rect->x;
            fflush(stdin);
            do {
                randdirect = rand()%4;
            }
            while(((randdirect == 0) && (*predirection == 1)) || ((randdirect == 1) && (*predirection == 0)) || ((randdirect == 2) && (*predirection == 3)) || ((randdirect == 3) && (*predirection == 2)));
            *predirection = randdirect;
			prof->animation.index_image = 0;
            break;
        default:
            break;
    }

}