/*
// Created by Lucile on 12/05/2021.
*/

#ifndef INSAGAME_TEST_COLLISION_H
#define INSAGAME_TEST_COLLISION_H

#include <gtest/gtest.h>
extern "C"{
#include "collision.h"
#include "map.h"
#include "personnage.h"
#include "prof.h"
#include "DS.h"
}

/*test_bounds_col*/

TEST(test_bounds_col,values_collision_gauche){
    Personnage personnage;
    init_pers(&personnage,-1,1,1,1,1);
    int res = test_bounds_col(&personnage);
    EXPECT_EQ(1,res);
}

TEST(test_bounds_col,values_collision_haut){
    Personnage personnage;
    init_pers(&personnage,1,-1,1,1,1);
    int res = test_bounds_col(&personnage);
    EXPECT_EQ(1,res);
}

TEST(test_bounds_col,values_collision_droite){
    Personnage personnage;
    init_pers(&personnage,WINDOW_WIDTH,1,1,1,1);
    int res = test_bounds_col(&personnage);
    EXPECT_EQ(1,res);
}

TEST(test_bounds_col,values_collision_bas){
    Personnage personnage;
    init_pers(&personnage,1, WINDOW_HEIGHT,1,1,1);
    int res = test_bounds_col(&personnage);
    EXPECT_EQ(1,res);
}

TEST(test_bounds_col,values_no_collision){
    Personnage personnage;
    init_pers(&personnage,1,1,1,1,1);
    int res = test_bounds_col(&personnage);
    EXPECT_EQ(0,res);
}

/*test_point_with_prof*/

TEST(test_point_with_prof, values_collision_prof){
    Prof prof;
    init_prof(&prof,10,10,5,5,1);
    int res = test_point_with_prof(11,12,&prof);
    EXPECT_EQ(1,res);
}

TEST(test_point_with_prof, values_no_collision_prof){
    Prof prof;
    init_prof(&prof,10,10,5,5,1);
    int res = test_point_with_prof(9,7,&prof);
    EXPECT_EQ(0,res);
}

TEST(test_point_with_prof, values_no_collision_prof_limits){
    Prof prof;
    init_prof(&prof,10,10,5,5,1);
    int res = test_point_with_prof(15,12,&prof);
    EXPECT_EQ(0,res);
}

/*test_point_with_map*/

TEST(test_point_with_map, values_collision_exam_points){
    MAP map; DS exam;
    init_map(&map,"../ressource/colored.bmp","../ressource/map_play.bmp");
    init_DS(&exam, 6, "../ressource/DS.png");
        //exam->x_DS = 237 exam->y_DS= 98 exam->larg_DS= 11 exam->long_DS= 19;
    int res = test_point_with_map(240, 100, &map, &exam);
	free_map(&map);
	free_DS(&exam);
    EXPECT_EQ(2,res);
}

TEST(test_point_with_map, values_collision_wall){
    MAP map; DS exam;
    init_map(&map,"../ressource/colored.bmp","../ressource/map_play.bmp");
    init_DS(&exam, 6, "../ressource/DS.png");
    int res = test_point_with_map(4, 25, &map, &exam);
	free_map(&map);
	free_DS(&exam);
    EXPECT_EQ(1,res);
}

TEST(test_point_with_map, values_collision_map_none){
    MAP map; DS exam;
    init_map(&map,"../ressource/colored.bmp","../ressource/map_play.bmp");
    init_DS(&exam, 6, "../ressource/DS.png");
    int res = test_point_with_map(4, 2, &map, &exam);
	free_map(&map);
	free_DS(&exam);
    EXPECT_EQ(0,res);
}

/*test_bounds_prof*/

TEST(test_bounds_prof,values_collision_gauche){
    Prof teacher;
    init_prof(&teacher,-1,1,1,1,1);
    int res = test_bounds_prof(&teacher);
    EXPECT_EQ(1,res);
}

TEST(test_bounds_prof,values_collision_haut){
    Prof teacher;
    init_prof(&teacher,1,-1,1,1,1);
    int res = test_bounds_prof(&teacher);
    EXPECT_EQ(1,res);
}

TEST(test_bounds_prof,values_collision_droite){
    Prof teacher;
    init_prof(&teacher,WINDOW_WIDTH,1,1,1,1);
    int res = test_bounds_prof(&teacher);
    EXPECT_EQ(1,res);
}

TEST(test_bounds_prof,values_collision_bas){
    Prof teacher;
    init_prof(&teacher,1, WINDOW_HEIGHT,1,1,1);
    int res = test_bounds_prof(&teacher);
    EXPECT_EQ(1,res);
}

TEST(test_bounds_prof,values_no_collision){
    Prof teacher;
    init_prof(&teacher,1,1,1,1,1);
    int res = test_bounds_prof(&teacher);
    EXPECT_EQ(0,res);
}

/*test_prof_col*/

TEST(test_prof_col,values_collision){
    Personnage personnage;
    init_pers(&personnage,10,10,3,3,1);
    Prof prof;
    init_prof(&prof, 9, 9, 4, 4, 1);
    int res = test_prof_col(&personnage, &prof);
    EXPECT_EQ(1,res);
}

TEST(test_prof_col,values_no_collision){
    Personnage personnage;
    init_pers(&personnage,10,10,3,3,1);
    Prof prof;
    init_prof(&prof, 5, 5, 4, 4, 1);
    int res = test_prof_col(&personnage, &prof);
    EXPECT_EQ(0,res);
}

TEST(test_prof_col,values_no_collision_limits){
    Personnage personnage;
    init_pers(&personnage,10,10,3,3,1);
    Prof prof;
    init_prof(&prof, 13, 13, 4, 4, 1);
    int res = test_prof_col(&personnage, &prof);
    EXPECT_EQ(0,res);
}

/*test_map_col*/

TEST(test_map_col,values_collision_exam){

    MAP map; DS exam; Personnage character; int res;
    init_map(&map,"../ressource/colored.bmp","../ressource/map_play.bmp");
    init_DS(&exam, 6, "../ressource/DS.png");
    init_pers(&character, 235,98, 5, 5,1);
    calcul_points_pers(&character);
    res = test_map_col(&character, &map, &exam);
	free_DS(&exam);
	free_map(&map);
    EXPECT_EQ(2,res);

}

TEST(test_map_col, values_collision_wall){

    Personnage personnage; MAP map; DS exam;
    init_pers(&personnage, 53, 20, 5, 5, 1);
    calcul_points_pers(&personnage);
    init_map(&map,"../ressource/colored.bmp","../ressource/map_play.bmp");
    init_DS(&exam, 6, "../ressource/DS.png");
    int res = test_map_col(&personnage,&map,&exam);
	free_DS(&exam);
	free_map(&map);
    EXPECT_EQ(1,res);

}

TEST(test_map_col, values_collision_none){

    Personnage personnage; MAP map; DS exam;
    init_pers(&personnage, 5,5, 5, 5,1);
    calcul_points_pers(&personnage);
    init_map(&map,"../ressource/colored.bmp","../ressource/map_play.bmp");
    init_DS(&exam, 6, "../ressource/DS.png");
    int res = test_map_col(&personnage,&map,&exam);
	free_DS(&exam);
	free_map(&map);
    EXPECT_EQ(0,res);

}


/*test_map_col_prof*/

TEST(test_map_col_prof, values_collision){
    int res; Prof teacher; MAP map; DS exam;
    init_prof(&teacher, 53, 20, 5, 5, 1);
    calcul_points_prof(&teacher);
    init_map(&map,"../ressource/colored.bmp","../ressource/map_play.bmp");
    init_DS(&exam, 6, "../ressource/DS.png");
    res = test_map_col_prof(&teacher, &map, &exam);
	free_DS(&exam);
	free_map(&map);
    EXPECT_EQ(1,res);

}

TEST(test_map_col_prof, values_no_collision_exam) {
    MAP map; DS exam; Prof teacher; int res;
    init_map(&map, "../ressource/colored.bmp", "../ressource/map_play.bmp");
    init_DS(&exam, 6, "../ressource/DS.png");
    init_prof(&teacher, 240, 100, 5, 5, 1);
    calcul_points_prof(&teacher);
    res = test_map_col_prof(&teacher, &map, &exam);
	free_DS(&exam);
	free_map(&map);
    EXPECT_EQ(0, res);
}

TEST(test_map_col_prof,values_collision_none_map_cole){
    Prof teacher; MAP map; DS exam;
    init_prof(&teacher, 5,5, 5, 5,1);
    calcul_points_prof(&teacher);
    init_map(&map,"../ressource/colored.bmp","../ressource/map_play.bmp");
    init_DS(&exam, 6, "../ressource/DS.png");
    int res = test_map_col_prof(&teacher,&map,&exam);
	free_DS(&exam);
	free_map(&map);
    EXPECT_EQ(0,res);
}

/*test_position*/

TEST(test_position,values_nothing){

    Personnage personnage; Prof prof; DS exam; MAP map; int res;
    init_map(&map,"../ressource/colored.bmp","../ressource/map_play.bmp");
    init_DS(&exam, 6, "../ressource/DS.png");
    init_pers(&personnage,5,5,5,5,1);
    init_prof(&prof,20,5,5,5,1);
    calcul_points_prof(&prof);
    calcul_points_pers(&personnage);
    res = test_position(&personnage, &prof, &exam, &map);
	free_DS(&exam);
	free_map(&map);
    EXPECT_EQ(0,res);

}

TEST(test_position,values_wall){
    Personnage personnage; Prof prof; DS exam; MAP map; int res;
    init_map(&map,"../ressource/colored.bmp","../ressource/map_play.bmp");
    init_DS(&exam, 6, "../ressource/DS.png");
    init_pers(&personnage, 53, 20, 5, 5, 1);
    init_prof(&prof,5,5,5,5,1);
    calcul_points_prof(&prof);
    calcul_points_pers(&personnage);
    res = test_position(&personnage, &prof, &exam, &map);
	free_DS(&exam);
	free_map(&map);
    EXPECT_EQ(1,res);
}

TEST(test_position,values_exam){

    Personnage personnage; Prof prof; DS exam; MAP map; int res;
    init_map(&map,"../ressource/colored.bmp","../ressource/map_play.bmp");
    init_DS(&exam, 6, "../ressource/DS.png");
    init_pers(&personnage, 235, 98, 5, 5, 1);
    init_prof(&prof,5,5,5,5,1);
    calcul_points_prof(&prof);
    calcul_points_pers(&personnage);
    res = test_position(&personnage, &prof, &exam, &map);
	free_DS(&exam);
	free_map(&map);
    EXPECT_EQ(2,res);

}

TEST(test_position,values_prof){

    Personnage personnage; Prof prof; DS exam; MAP map; int res;
    init_map(&map,"../ressource/colored.bmp","../ressource/map_play.bmp");
    init_DS(&exam, 6, "../ressource/DS.png");
    init_pers(&personnage, 5,5,5,5,1);
    init_prof(&prof,7,7,5,5,1);
    calcul_points_prof(&prof);
    calcul_points_pers(&personnage);
    res = test_position(&personnage, &prof, &exam, &map);
	free_DS(&exam);
	free_map(&map);
    EXPECT_EQ(3,res);

}

TEST(test_position,values_bound){

    Personnage personnage; Prof prof; DS exam; MAP map; int res;
    init_map(&map,"../ressource/colored.bmp","../ressource/map_play.bmp");
    init_DS(&exam, 6, "../ressource/DS.png");
    init_pers(&personnage, -1,1,5,5,1);
    init_prof(&prof,7,7,5,5,1);
    calcul_points_prof(&prof);
    calcul_points_pers(&personnage);
    res = test_position(&personnage, &prof, &exam, &map);
	free_map(&map);
	free_DS(&exam);
    EXPECT_EQ(4,res);

}

#endif /*INSAGAME_TEST_COLLISION_H*/
